/* 
 * Copyright (C) 2005 Darren Hutchinson (dbh@gbdt.com.au)
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA. 
 *
 * $Id: button.c,v 1.4 2005/05/29 06:26:33 dbh Exp $
 */

#include <inttypes.h>
#include <avr/io.h>

#include "consts.h"
#include "button.h"

#define BUT_MASK        (BUT_LEFT | BUT_RIGHT)

/* buttonInit() initializes the button input bits and the state for each
 * button
 *
 * Passed:
 *         Nothing
 *
 * Returns:
 *         Nothing.
 */
void
buttonInit(void)
{
    // Setup the bits as inputs w/ pullup
    BUT_DDR &= ~BUT_MASK;
    BUT_PORT |= BUT_MASK;
}

/* buttonRead() reads the current button settings
 *
 * Passed
 *      nothing
 *
 * Returns
 *      BUT_LEFT if left button pressed
 *      BUT_RIGHT if right button pressed
 *      (BUT_LEFT | BUT_RIGHT) if both pressed
 */
uint8_t
buttonRead(void)
{
    return ~BUT_PIN & BUT_MASK;
}

