/* 
 * Copyright (C) 2004 Darren Hutchinson (dbh@gbdt.com.au)
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA. 
 *
 * $Id: serial.h,v 1.3 2005/06/01 00:15:12 dbh Exp $
 */

#ifndef _SERIAL_H_
#define _SERIAL_H_

#include <avr/io.h>
#include <avr/pgmspace.h>

// Define the length of a robofocus command (exc leading F, cmd, and csum)
#define CDLEN  6

// Define the serial command timeout
#define RX_TIMEOUT      100

/* The following structure is used to hold Robofocus commands
 */
typedef struct
{
    volatile int8_t      pos;   // Next read/write position
    char        cmd;            // Command (first char after leading 'F'
    char        data[CDLEN];    // Data associated with command
    char        csum;           // Checksum character
} message_t;

/* Functions in serial.c */
void serialInit(void);
void serialPoll(void);
void serialTxMsg(char cmd, char data[CDLEN]);
void serialTxChar(char ch);

message_t *serialRxMsg(void);
void serialRxDone(message_t *pMsg);
uint8_t serialWasChar(void);

void print8(uint8_t num);
void print16(uint16_t num);
void print32(uint32_t num);

#endif /* _SERIAL_H_ */
